import re

from cookiecutter.main import cookiecutter
from github import Github


def project_name(safe_repo_name: str) -> str:
    return " ".join(word.capitalize() for word in safe_repo_name.split("-"))


def sanitize_repo_name(unsafe_repo_name: str) -> str:
    word = unsafe_repo_name.strip(" -")
    word = re.sub(r"[\ \_]+", "-", word)
    word = re.sub("([A-Z]+)([A-Z][a-z])", r"\1-\2", word)
    word = re.sub(r"([a-z\d])([A-Z])", r"\1-\2", word)
    return word.lower()


def package_name(safe_repo_name: str) -> str:
    return safe_repo_name.replace("-", "_")


class Context:
    def __init__(self, github: Github, repo_name: str, src_dir: str) -> None:
        self._user = github.get_user()
        self.repo_name = sanitize_repo_name(repo_name)
        self._project_name = project_name(safe_repo_name=self.repo_name)
        self._package_name = package_name(safe_repo_name=self.repo_name)
        self._src_dir = src_dir

    def dict(self) -> dict[str, str]:
        return {
            "author_email": self._user.email,
            "author_name": self._user.name,
            "git_registry": "https://github.com/",
            "git_registry_account": self._user.login,
            "package_name": self._package_name,
            "project_name": self._project_name,
            "repo_name": self.repo_name,
            "src_dir": self._src_dir,
        }


def generate_template(template_url: str, context: Context) -> None:
    cookiecutter(
        template=template_url,
        extra_context=context.dict(),
        no_input=True,
    )



from github import Github
import typer

from racoon.argument_types import DefaultSrcDir
from racoon.argument_types import DefaultTemplateURL
from racoon.argument_types import RequiredAccessToken
from racoon.git import init_template
from racoon.github_integration import GitHubIntegration
from racoon.template_generation import Context
from racoon.template_generation import generate_template

app = typer.Typer()


@app.command()
def generate(
    project_name: str,
    github_access_token: str = RequiredAccessToken,
    src_dir: str = DefaultSrcDir,
    template_url: str = DefaultTemplateURL,
) -> None:
    github = Github(login_or_token=github_access_token)
    config = Context(github=github, repo_name=project_name, src_dir=src_dir)
    generate_template(template_url=template_url, context=config)
    github_integration = GitHubIntegration(github=github)
    repo = github_integration.setup(repo_name=config.repo_name)
    init_template(repository=repo)


if __name__ == "__main__":
    app()



import typer

RequiredAccessToken: str = typer.Option(..., envvar="GITHUB_ACCESS_TOKEN")
DefaultSrcDir: str = typer.Option("src")
DefaultTemplateURL: str = typer.Option("https://github.com/nymann/python-template.git")
